@@ -1,4 +1,4 @@
-# Copyright 2014, Ansible, Inc.
+# Copyright 2015, Ansible, Inc.
 # Luke Sneeringer <lsneeringer@ansible.com>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,13 +17,15 @@ import contextlib
 import copy
 import functools
 import json
+import warnings
 
-from requests.exceptions import ConnectionError
+from requests.exceptions import ConnectionError, SSLError
 from requests.sessions import Session
 from requests.models import Response
+from requests.packages import urllib3
 
 from tower_cli.conf import settings
-from tower_cli.utils import data_structures, debug, exceptions as exc
+from tower_cli.utils import data_structures, debug, secho, exceptions as exc
 
 
 class Client(Session):
@@ -50,6 +52,11 @@ class Client(Session):
         host = settings.host
         if '://' not in host:
             host = 'https://%s' % host.strip('/')
+        elif host.startswith('http://') and settings.verify_ssl:
+            raise exc.TowerCLIError(
+                'Can not verify ssl with non-https protocol. Change the '
+                'verify_ssl configuration setting to continue.'
+            )
         return '%s/api/v1/' % host.rstrip('/')
 
     @functools.wraps(Session.request)
@@ -86,11 +93,40 @@ class Client(Session):
         if headers.get('Content-Type', '') == 'application/json':
             kwargs['data'] = json.dumps(kwargs.get('data', {}))
 
+        # Decide whether to require SSL verification
+        verify_ssl = True
+        if (settings.verify_ssl is False) or hasattr(settings, 'insecure'):
+            verify_ssl = False
+        elif settings.certificate is not None:
+            verify_ssl = settings.certificate
+
         # Call the superclass method.
         try:
-            r = super(Client, self).request(method, url, *args,
-                                            verify=False, **kwargs)
+            with warnings.catch_warnings():
+                warnings.simplefilter(
+                    "ignore", urllib3.exceptions.InsecureRequestWarning)
+                r = super(Client, self).request(
+                    method, url, *args, verify=verify_ssl, **kwargs)
+        except SSLError as ex:
+            # Throw error if verify_ssl not set to false and server
+            #  is not using verified certificate.
+            if settings.verbose:
+                debug.log('SSL connection failed:', fg='yellow', bold=True)
+                debug.log(str(ex), fg='yellow', bold=True, nl=2)
+            if not settings.host.startswith('http'):
+                secho('Suggestion: add the correct http:// or '
+                      'https:// prefix to the host configuration.',
+                      fg='blue', bold=True)
+            raise exc.ConnectionError(
+                'Could not establish a secure connection. '
+                'Please add the server to your certificate '
+                'authority.\nYou can run this command without verifying SSL '
+                'with the --insecure flag, or permanently disable '
+                'verification by the config setting:\n\n '
+                'tower-cli config verify_ssl false'
+            )
         except ConnectionError as ex:
+            # Throw error if server can not be reached.
             if settings.verbose:
                 debug.log('Cannot connect to Tower:', fg='yellow', bold=True)
                 debug.log(str(ex), fg='yellow', bold=True, nl=2)
@@ -175,7 +211,7 @@ class Client(Session):
             faux_adapter = FauxAdapter(
                 url_pattern=self.prefix.rstrip('/') + '%s',
             )
-            
+
             try:
                 self.adapters.clear()
                 self.mount('https://', faux_adapter)
@@ -183,7 +219,7 @@ class Client(Session):
                 yield faux_adapter
             finally:
                 self.adapters = adapters
-            
+
 
 class APIResponse(Response):
     """A Response subclass which preseves JSON key order (but makes no other
