@@ -1,4 +1,4 @@
-# Copyright 2014, Ansible, Inc.
+# Copyright 2015, Ansible, Inc.
 # Luke Sneeringer <lsneeringer@ansible.com>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +14,7 @@
 # limitations under the License.
 
 import os
+import stat
 import warnings
 
 import six
@@ -21,7 +22,7 @@ import six
 import click
 
 from tower_cli.conf import Parser, settings
-from tower_cli.utils import exceptions as exc
+from tower_cli.utils import exceptions as exc, secho
 
 
 # Note: This uses `click.command`, not `tower_cli.utils.decorators.command`,
@@ -34,12 +35,12 @@ from tower_cli.utils import exceptions as exc
                    'Probably will require sudo.\n'
                    'Deprecated: Use `--scope=global` instead.')
 @click.option('--scope', type=click.Choice(['local', 'user', 'global']),
-                         default='user',
-                         help='The config file to write. '
-                              '"local" writes to a config file in the local '
-                              'directory; "user" writes to the home directory,'
-                              ' and "global" to a system-wide directory '
-                              '(probably requires sudo).')
+              default='user',
+              help='The config file to write. '
+                   '"local" writes to a config file in the local '
+                   'directory; "user" writes to the home directory,'
+                   ' and "global" to a system-wide directory '
+                   '(probably requires sudo).')
 @click.option('--unset', is_flag=True,
               help='Remove reference to this configuration option from '
                    'the config file.')
@@ -47,10 +48,10 @@ def config(key=None, value=None, scope='user', global_=False, unset=False):
     """Read or write tower-cli configuration.
 
     `tower config` saves the given setting to the appropriate Tower CLI;
-    either the user's ~/.tower_cli.cfg file, or the /etc/awx/tower_cli.cfg
+    either the user's ~/.tower_cli.cfg file, or the /etc/tower/tower_cli.cfg
     file if --global is used.
 
-    Writing to /etc/awx/tower_cli.cfg is likely to require heightened
+    Writing to /etc/tower/tower_cli.cfg is likely to require heightened
     permissions (in other words, sudo).
     """
     # If the old-style `global_` option is set, issue a deprecation notice.
@@ -72,7 +73,7 @@ def config(key=None, value=None, scope='user', global_=False, unset=False):
             'user': 'User options (set with `tower-cli config`; stored in '
                     '~/.tower_cli.cfg).',
             'global': 'Global options (set with `tower-cli config '
-                      '--scope=global`, stored in /etc/awx/tower_cli.cfg).',
+                      '--scope=global`, stored in /etc/tower/tower_cli.cfg).',
             'defaults': 'Defaults.',
         }
 
@@ -93,8 +94,7 @@ def config(key=None, value=None, scope='user', global_=False, unset=False):
 
             # Print a segment header
             if will_echo:
-                click.secho('# %s' % parser_desc[name],
-                            fg='green', bold=True)
+                secho('# %s' % parser_desc[name], fg='green', bold=True)
 
             # Iterate over each option in the parser and, if we haven't
             # already seen an option at higher precedence, print it.
@@ -129,10 +129,10 @@ def config(key=None, value=None, scope='user', global_=False, unset=False):
     # First, we need the appropriate file.
     filename = os.path.expanduser('~/.tower_cli.cfg')
     if scope == 'global':
-        if not os.path.isdir('/etc/awx/'):
-            raise exc.TowerCLIError('/etc/awx/ does not exist, and this '
+        if not os.path.isdir('/etc/tower/'):
+            raise exc.TowerCLIError('/etc/tower/ does not exist, and this '
                                     'command cowardly declines to create it.')
-        filename = '/etc/awx/tower_cli.cfg'
+        filename = '/etc/tower/tower_cli.cfg'
     elif scope == 'local':
         filename = '.tower_cli.cfg'
 
@@ -147,14 +147,24 @@ def config(key=None, value=None, scope='user', global_=False, unset=False):
         parser.set('general', key, value)
     with open(filename, 'w') as config_file:
         parser.write(config_file)
+
+    # Give rw permissions to user only fix for issue number 48
+    try:
+        os.chmod(filename, stat.S_IRUSR | stat.S_IWUSR)
+    except Exception as e:
+        warnings.warn(
+            'Unable to set permissions on {0} - {1} '.format(filename, e),
+            UserWarning
+            )
     click.echo('Configuration updated successfully.')
 
 
 def echo_setting(key):
     """Echo a setting to the CLI."""
     value = getattr(settings, key)
-    click.secho('%s: ' % key, fg='magenta', bold=True, nl=False)
-    click.secho(six.text_type(value),
+    secho('%s: ' % key, fg='magenta', bold=True, nl=False)
+    secho(
+        six.text_type(value),
         bold=True,
         fg='white' if isinstance(value, six.text_type) else 'cyan',
     )
