@@ -1,4 +1,4 @@
-# Copyright 2014, Ansible, Inc.
+# Copyright 2015, Ansible, Inc.
 # Luke Sneeringer <lsneeringer@ansible.com>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,14 +16,13 @@
 import contextlib
 import copy
 import os
+import stat
 import warnings
 
 import six
 from six.moves import configparser
 from six import StringIO
 
-from sdict import adict
-
 
 class Parser(configparser.ConfigParser):
     """ConfigParser subclass that doesn't strictly require section
@@ -37,6 +36,15 @@ class Parser(configparser.ConfigParser):
         """
         # Attempt to read the file using the superclass implementation.
         #
+        # Check the permissions of the file we are considering reading
+        # if the file exists and the permissions expose it to reads from
+        # other users, raise a warning
+        if os.path.isfile(fpname):
+            file_permission = os.stat(fpname)
+            if (file_permission.st_mode & stat.S_IRGRP) or \
+               (file_permission.st_mode & stat.S_IROTH):
+                warnings.warn('File {0} readable by group or others.'
+                              .format(fpname), RuntimeWarning)
         # If it doesn't work because there's no section header, then
         # create a section header and call the superclass implementation
         # again.
@@ -56,7 +64,7 @@ class Settings(object):
     The order of precedence for settings, from least to greatest, is:
 
         - defaults provided in this method
-        - `/etc/awx/tower_cli.cfg`
+        - `/etc/tower/tower_cli.cfg`
         - `~/.tower_cli.cfg`
         - command line arguments
 
@@ -72,11 +80,15 @@ class Settings(object):
         # Initialize the data dictionary for the default level
         # precedence (that is, the bottom of the totem pole).
         defaults = {
+            'color': 'true',
             'format': 'human',
             'host': '127.0.0.1',
             'password': '',
             'username': '',
+            'verify_ssl': 'true',
             'verbose': 'false',
+            'description_on': 'false',
+            'certificate': '',
         }
         self._defaults = Parser(defaults=defaults)
         self._defaults.add_section('general')
@@ -84,10 +96,10 @@ class Settings(object):
         # If there is a global settings file, initialize it.
         self._global = Parser()
         self._global.add_section('general')
-        if os.path.isdir('/etc/awx/'):
-            # Sanity check: Try to actually get a list of files in `/etc/awx/`.
+        if os.path.isdir('/etc/tower/'):
+            # Sanity check: Try to get a list of files in `/etc/tower/`.
             #
-            # The default Tower installation caused `/etc/awx/` to have
+            # The default Tower installation caused `/etc/tower/` to have
             # extremely restrictive permissions, since it has its own user
             # and group and has a chmod of 0750.
             #
@@ -98,16 +110,16 @@ class Settings(object):
             # Therefore, check for that particular problem and give a warning
             # if we're in that situation.
             try:
-                global_settings = 'tower_cli.cfg' in os.listdir('/etc/awx/')
+                os.listdir('/etc/tower/')
             except OSError:
-                warnings.warn('/etc/awx/ is present, but not readable with '
+                warnings.warn('/etc/tower/ is present, but not readable with '
                               'current permissions. Any settings defined in '
-                              '/etc/awx/tower_cli.cfg will not be honored.',
+                              '/etc/tower/tower_cli.cfg will not be honored.',
                               RuntimeWarning)
 
             # If there is a global settings file for Tower CLI, read in its
             # contents.
-            self._global.read('/etc/awx/tower_cli.cfg')
+            self._global.read('/etc/tower/tower_cli.cfg')
 
         # Initialize a parser for the user settings file.
         self._user = Parser()
@@ -134,7 +146,7 @@ class Settings(object):
 
             # Sanity check: if this directory corresponds to our global or
             # user directory, skip it.
-            if local_dir in (os.path.expanduser('~'), '/etc/awx'):
+            if local_dir in (os.path.expanduser('~'), '/etc/tower'):
                 continue
 
             # Add this directory to the list.
@@ -203,6 +215,8 @@ class Settings(object):
         """Temporarily override the runtime settings, which exist at the
         highest precedence level.
         """
+        kwargs = config_from_environment(kwargs)
+
         # Coerce all values to strings (to be coerced back by configparser
         # later) and defenestrate any None values.
         for k, v in copy.copy(kwargs).items():
@@ -239,6 +253,22 @@ class Settings(object):
                 self._cache.pop(k, None)
 
 
+def config_from_environment(kwargs):
+    """Read tower-cli config values from the environment if present, being
+    careful not to override config values that were explicitly passed in.
+    """
+    CONFIG_OPTIONS = ('host', 'username', 'password', 'verify_ssl', 'format',
+                      'color', 'verbose', 'description_on', 'certificate')
+    kwargs = copy.copy(kwargs)
+    for k in CONFIG_OPTIONS:
+        if k not in kwargs or kwargs[k] is None:
+            env = 'TOWER_' + k.upper()
+            v = os.getenv(env, None)
+            if v is not None:
+                kwargs[k] = v
+    return kwargs
+
+
 # The primary way to interact with settings is to simply hit the
 # already constructed settings object.
 settings = Settings()
