@@ -1,4 +1,4 @@
-# Copyright 2014, Ansible, Inc.
+# Copyright 2015, Ansible, Inc.
 # Luke Sneeringer <lsneeringer@ansible.com>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,18 +17,19 @@ from __future__ import absolute_import, division, unicode_literals
 
 import functools
 import inspect
+import itertools
 import json
+import yaml
 import math
 import re
 import sys
+import time
 from copy import copy
 
 import six
 
-from requests.compat import quote
-
 import click
-from click.decorators import _make_command
+from click._compat import isatty as is_tty
 
 from tower_cli import resources
 from tower_cli.api import client
@@ -36,10 +37,10 @@ from tower_cli.conf import settings
 from tower_cli.models.fields import Field
 from tower_cli.utils import exceptions as exc
 from tower_cli.utils.command import Command
-from tower_cli.utils import debug
+from tower_cli.utils import debug, secho
 from tower_cli.utils.data_structures import OrderedDict
 from tower_cli.utils.decorators import command
-from tower_cli.utils.types import File
+from tower_cli.utils.parser import ordered_dump
 
 
 class ResourceMeta(type):
@@ -54,13 +55,19 @@ class ResourceMeta(type):
         for base in bases:
             base_commands = getattr(base, 'commands', [])
             commands = commands.union(base_commands)
+
+        # Read list of deprecated resource methods if present.
+        deprecates = attrs.pop('deprecated_methods', [])
+
         for key, value in attrs.items():
             if getattr(value, '_cli_command', False):
                 commands.add(key)
+                if key in deprecates:
+                    setattr(value, 'deprecated', True)
 
             # If this method has been overwritten from the superclass, copy
             # any click options or arguments from the superclass implementation
-            # down tot the subclass implementation.
+            # down to the subclass implementation.
             if not len(bases):
                 continue
             superclass = bases[0]
@@ -119,12 +126,11 @@ class ResourceMeta(type):
 
 class BaseResource(six.with_metaclass(ResourceMeta)):
     """Abstract class representing resources within the Ansible Tower
-    system, on which actions can be taken.
-    """
+    system, on which actions can be taken."""
     abstract = True  # Not inherited.
     cli_help = ''
     endpoint = None
-    unique_criterion = 'name'
+    identity = ('name',)
 
     def as_command(self):
         """Return a `click.Command` class for interacting with this
@@ -136,10 +142,13 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
             """
             def __init__(self, resource, *args, **kwargs):
                 self.resource = resource
-                self.resource_name = getattr(resource, 'resource_name',
-                                            resource.__module__.split('.')[-1])
+                self.resource_name = getattr(
+                    resource, 'resource_name',
+                    resource.__module__.split('.')[-1]
+                )
                 self.resource_name = self.resource_name.replace('_', ' ')
-                super(Subcommand, self).__init__(*args,
+                super(Subcommand, self).__init__(
+                    *args,
                     help=self.resource.cli_help,
                     **kwargs
                 )
@@ -155,12 +164,10 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                 decorate it as a click command, and return that method.
                 """
                 # Sanity check: Does a method exist corresponding to this
-                # command? If not, this is an error.
+                # command? If not, None is returned for click to raise
+                # exception.
                 if not hasattr(self.resource, name):
-                    raise exc.UsageError(
-                        'The %s resource has no such command: "%s"' %
-                        (self.resource_name, name),
-                    )
+                    return None
 
                 # Get the method.
                 method = getattr(self.resource, name)
@@ -198,11 +205,18 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                 new_method.__click_params__ = copy(click_params)
 
                 # Write options based on the fields available on this resource.
-                if attrs.pop('use_fields_as_options', True):
+                fao = attrs.pop('use_fields_as_options', True)
+                if fao:
                     for field in reversed(self.resource.fields):
                         if not field.is_option:
                             continue
 
+                        # If we got an iterable rather than a boolean,
+                        # then it is a list of fields to use; check for
+                        # presence in that list.
+                        if not isinstance(fao, bool) and field.name not in fao:
+                            continue
+
                         # Create the initial arguments based on the
                         # option value. If we have a different key to use
                         # (which is what gets routed to the Tower API),
@@ -211,13 +225,28 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                         if field.key:
                             args.insert(0, field.key)
 
+                        # short name aliases for common flags
+                        short_fields = {
+                            'name': 'n',
+                            'description': 'd',
+                            'inventory': 'i',
+                            'extra_vars': 'e'
+                        }
+                        if field.name in short_fields:
+                            args.append('-'+short_fields[field.name])
+
                         # Apply the option to the method.
-                        click.option(*args,
+                        option_help = field.help
+                        if field.required:
+                            option_help = '[REQUIRED] ' + option_help
+                        click.option(
+                            *args,
                             default=field.default if not ignore_defaults
-                                                  else None,
-                            help=field.help,
+                            else None,
+                            help=option_help,
                             type=field.type,
-                            show_default=field.show_default
+                            show_default=field.show_default,
+                            multiple=field.multiple
                         )(new_method)
 
                 # Make a click Command instance using this method
@@ -229,7 +258,7 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                 code = six.get_function_code(method)
                 if 'pk' in code.co_varnames:
                     click.argument('pk', nargs=1, required=False,
-                                         type=int, metavar='[ID]')(cmd)
+                                   type=int, metavar='[ID]')(cmd)
 
                 # Done; return the command.
                 return cmd
@@ -272,6 +301,11 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                 """
                 @functools.wraps(method)
                 def func(*args, **kwargs):
+                    # Echo warning if this method is deprecated.
+                    if getattr(method, 'deprecated', False):
+                        debug.log('This method is deprecated in Tower 3.0.',
+                                  header='warning')
+
                     result = method(*args, **kwargs)
 
                     # If this was a request that could result in a modification
@@ -288,7 +322,7 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                     output = format(result)
 
                     # Perform the echo.
-                    click.secho(output, **color_info)
+                    secho(output, **color_info)
                 return func
 
             def _format_json(self, payload):
@@ -297,6 +331,13 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                 """
                 return json.dumps(payload, indent=2)
 
+            def _format_yaml(self, payload):
+                """Convert the payload into a YAML string with proper
+                indentation and return it.
+                """
+                return ordered_dump(payload, Dumper=yaml.SafeDumper,
+                                    default_flow_style=False)
+
             def _format_human(self, payload):
                 """Convert the payload into an ASCII table suitable for
                 printing on screen and return it.
@@ -306,11 +347,13 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
 
                 # What are the columns we will show?
                 columns = [field.name for field in self.resource.fields
-                                      if field.display]
+                           if field.display or settings.description_on and
+                           field.name == 'description']
                 columns.insert(0, 'id')
 
                 # Sanity check: If there is a "changed" key in our payload
                 # and little else, we print a short message and not a table.
+                # this specifically applies to deletion
                 if 'changed' in payload and 'id' not in payload:
                     return 'OK. (changed: {0})'.format(
                         six.text_type(payload['changed']).lower(),
@@ -319,6 +362,8 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                 # Sanity check: If there is no ID and no results, then this
                 # is unusual output; keep our table formatting, but plow
                 # over the columns-as-keys stuff above.
+                # this originally applied to launch/status/update methods
+                # but it may become deprecated
                 if 'id' not in payload and 'results' not in payload:
                     columns = [i for i in payload.keys()]
 
@@ -346,7 +391,8 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                 for col in columns:
                     widths[col] = max(
                         len(col),
-                        *[len(six.text_type(i[col])) for i in raw_rows]
+                        *[len(six.text_type(i.get(col, 'N/A')))
+                          for i in raw_rows]
                     )
 
                 # It's possible that the column widths will exceed our terminal
@@ -375,8 +421,8 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                     data_row = ''
                     for col in columns:
                         template = '{0:%d}' % widths[col]
-                        value = raw_row[col]
-                        if isinstance(raw_row[col], bool):
+                        value = raw_row.get(col, 'N/A')
+                        if isinstance(raw_row.get(col, 'N/A'), bool):
                             template = template.replace('{0:', '{0:>')
                             value = six.text_type(value).lower()
                         data_row += template.format(value or '') + ' '
@@ -390,15 +436,21 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                 ))
                 if page:
                     response += '(Page %d of %d.)' % (page, total_pages)
+                if payload.get('changed', False):
+                    response = 'Resource changed.\n' + response
                 return response
 
         return Subcommand(resource=self)
 
 
-class Resource(BaseResource):
+class ResourceMethods(BaseResource):
     """Abstract subclass of BaseResource that adds the standard create,
     modify, list, get, and delete methods.
-    """
+
+    Some of these methods are not created as commands, but will be
+    implemented as commands inside of non-abstract child classes.
+    Particularly, create is not a command in this class, but will be for
+    some (but not all) child classes."""
     abstract = True  # Not inherited.
 
     # The basic methods for interacting with a resource are `read`, `write`,
@@ -409,8 +461,18 @@ class Resource(BaseResource):
     # `get` and `list` are wrappers around `read` and `create` and
     # `modify` are wrappers around `write`.
 
+    def _pop_none(self, kwargs):
+        """Remove default values (anything where the value is None).
+        click is unfortunately bad at the way it sends through unspecified
+        defaults."""
+        for key, value in copy(kwargs).items():
+            if value is None:
+                kwargs.pop(key)
+            if hasattr(value, 'read'):
+                kwargs[key] = value.read()
+
     def read(self, pk=None, fail_on_no_results=False,
-                   fail_on_multiple_results=False, **kwargs):
+             fail_on_multiple_results=False, **kwargs):
         """Retrieve and return objects from the Ansible Tower API.
 
         If an `object_id` is provided, only attempt to read that object,
@@ -435,18 +497,18 @@ class Resource(BaseResource):
         queries = kwargs.pop('query', [])
 
         # Remove default values (anything where the value is None).
-        # click is unfortunately bad at the way it sends through unspecified
-        # defaults.
-        for key, value in copy(kwargs).items():
-            if value is None:
-                kwargs.pop(key)
-            if hasattr(value, 'read'):
-                kwargs[key] = value.read()
+        self._pop_none(kwargs)
+
+        # Remove fields that are specifically excluded from lookup
+        for field in self.fields:
+            if field.no_lookup and field.name in kwargs:
+                kwargs.pop(field.name)
 
         # If queries were provided, process them.
         for query in queries:
             if query[0] in kwargs:
-                raise exc.BadRequest('Attempted to set %s twice.' % query[0])
+                raise exc.BadRequest('Attempted to set %s twice.'
+                                     % query[0].replace('_', '-'))
             kwargs[query[0]] = query[1]
 
         # Make the request to the Ansible Tower API.
@@ -473,14 +535,16 @@ class Resource(BaseResource):
         # Did we get more than one result back?
         # If so, this is also an error, and we need to complain.
         if fail_on_multiple_results and resp['count'] >= 2:
-            raise exc.MultipleResults('Expected one result, got %d. Tighten '
+            raise exc.MultipleResults('Expected one result, got %d. Possibly '
+                                      'caused by not providing required '
+                                      'fields. Please tighten '
                                       'your criteria.' % resp['count'])
 
         # Return the response.
         return resp
 
     def write(self, pk=None, create_on_missing=False, fail_on_found=False,
-                    force_on_exists=True, **kwargs):
+              force_on_exists=True, **kwargs):
         """Modify the given object using the Ansible Tower API.
         Return the object and a boolean value informing us whether or not
         the record was changed.
@@ -504,13 +568,7 @@ class Resource(BaseResource):
         existing_data = {}
 
         # Remove default values (anything where the value is None).
-        # click is unfortunately bad at the way it sends through unspecified
-        # defaults.
-        for key, value in copy(kwargs).items():
-            if value is None:
-                kwargs.pop(key)
-            if hasattr(value, 'read'):
-                kwargs[key] = value.read()
+        self._pop_none(kwargs)
 
         # Determine which record we are writing, if we weren't given a
         # primary key.
@@ -537,7 +595,7 @@ class Resource(BaseResource):
         missing_fields = [i for i in required_fields if i not in kwargs]
         if missing_fields and not pk:
             raise exc.BadRequest('Missing required fields: %s' %
-                                 ', '.join(missing_fields))
+                                 ', '.join(missing_fields).replace('_', '-'))
 
         # Sanity check: Do we need to do a write at all?
         # If `force_on_exists` is False and the record was, in fact, found,
@@ -565,6 +623,11 @@ class Resource(BaseResource):
             answer.update(existing_data)
             return answer
 
+        # Reinsert None for special case of null association
+        for key in kwargs:
+            if kwargs[key] == 'null':
+                kwargs[key] = None
+
         # Get the URL and method to use for the write.
         url = self.endpoint
         method = 'POST'
@@ -642,8 +705,8 @@ class Resource(BaseResource):
                   help='If set, collate all pages of content from the API '
                        'when returning results.')
     @click.option('--page', default=1, type=int, show_default=True,
-                            help='The page to show. Ignored if --all-pages '
-                                 'is sent.')
+                  help='The page to show. Ignored if --all-pages '
+                       'is sent.')
     @click.option('-Q', '--query', required=False, nargs=2, multiple=True,
                   help='A key and value to be passed as an HTTP query string '
                        'key and value to the Tower API. Will be run through '
@@ -673,6 +736,9 @@ class Resource(BaseResource):
             if not response[key]:
                 continue
             match = re.search(r'page=(?P<num>[\d]+)', response[key])
+            if match is None and key == 'previous':
+                response[key] = 1
+                continue
             response[key] = int(match.groupdict()['num'])
 
         # If we were asked for all pages, keep retrieving pages until we
@@ -686,46 +752,6 @@ class Resource(BaseResource):
         # Done; return the response
         return response
 
-    @resources.command
-    @click.option('--fail-on-found', default=False,
-                  show_default=True, type=bool, is_flag=True,
-                  help='If used, return an error if a matching record already '
-                       'exists.')
-    @click.option('--force-on-exists', default=False,
-                  show_default=True, type=bool, is_flag=True,
-                  help='If used, if a match is found on unique fields, other '
-                       'fields will be updated to the provided values. If '
-                       'False, a match causes the request to be a no-op.')
-    def create(self, fail_on_found=False, force_on_exists=False, **kwargs):
-        """Create an object.
-
-        If unique fields exist and all unique fields are provided, and a match
-        is found, then no-op (unless `force_on_exists` is set) but do not
-        fail (unless `fail_on_found` is set).
-        """
-        return self.write(create_on_missing=True, fail_on_found=fail_on_found,
-                          force_on_exists=force_on_exists, **kwargs)
-
-    @resources.command
-    @click.option('--create-on-missing', default=False,
-                  show_default=True, type=bool, is_flag=True,
-                  help='If used, and if options rather than a primary key are '
-                       'used to attempt to match a record, will create the '
-                       'record if it does not exist. This is an alias to '
-                       '`create --force-on-exists`.')
-    def modify(self, pk=None, create_on_missing=False, **kwargs):
-        """Modify an already existing object.
-
-        If unique fields exist and are provided, they can be used in lieu of
-        a primary key for a lookup; in such a case, only non-unique fields
-        are written.
-
-        To modify unique fields, you must use the primary key for the lookup.
-        """
-        force_on_exists = kwargs.pop('force_on_exists', True)
-        return self.write(pk, create_on_missing=create_on_missing,
-                              force_on_exists=force_on_exists, **kwargs)
-
     def _assoc(self, url_fragment, me, other):
         """Associate the `other` record with the `me` record."""
 
@@ -759,7 +785,7 @@ class Resource(BaseResource):
         return {'changed': True}
 
     def _lookup(self, fail_on_missing=False, fail_on_found=False,
-                      include_debug_header=True, **kwargs):
+                include_debug_header=True, **kwargs):
         """Attempt to perform a lookup that is expected to return a single
         result, and return the record.
 
@@ -769,9 +795,9 @@ class Resource(BaseResource):
         # Determine which parameters we are using to determine
         # the appropriate field.
         read_params = {}
-        for unique_field in self.unique_fields:
-            if unique_field in kwargs:
-                read_params[unique_field] = kwargs[unique_field]
+        for field_name in self.identity:
+            if field_name in kwargs:
+                read_params[field_name] = kwargs[field_name]
 
         # Sanity check: Do we have any parameters?
         # If not, then there's no way for us to do this read.
@@ -789,7 +815,240 @@ class Resource(BaseResource):
                                 'you requested a failure in that case.' %
                                 read_params)
             return existing_data
-        except exc.NotFound as ex:
+        except exc.NotFound:
             if fail_on_missing:
-                raise
+                raise exc.NotFound('A record matching %s does not exist, and '
+                                   'you requested a failure in that case.' %
+                                   read_params)
             return {}
+
+
+class MonitorableResource(ResourceMethods):
+    """A resource that is able to be tied to a running task, such as a job
+    or project, and thus able to be monitored.
+    """
+    abstract = True  # Not inherited.
+
+    def status(self, pk, detail=False):
+        """A stub method requesting the status of the resource."""
+        raise NotImplementedError('This resource does not implement a status '
+                                  'method, and must do so.')
+
+    @resources.command
+    @click.option('--min-interval',
+                  default=1, help='The minimum interval to request an update '
+                                  'from Tower.')
+    @click.option('--max-interval',
+                  default=30, help='The maximum interval to request an update '
+                                   'from Tower.')
+    @click.option('--timeout', required=False, type=int,
+                  help='If provided, this command (not the job) will time out '
+                       'after the given number of seconds.')
+    def monitor(self, pk, min_interval=1, max_interval=30,
+                timeout=None, outfile=sys.stdout, **kwargs):
+        """Monitor a running job.
+
+        Blocks further input until the job completes (whether successfully or
+        unsuccessfully) and a final status can be given.
+        """
+        dots = itertools.cycle([0, 1, 2, 3])
+        longest_string = 0
+        interval = min_interval
+        start = time.time()
+
+        # Poll the Ansible Tower instance for status, and print the status
+        # to the outfile (usually standard out).
+        #
+        # Note that this is one of the few places where we use `secho`
+        # even though we're in a function that might theoretically be imported
+        # and run in Python.  This seems fine; outfile can be set to /dev/null
+        # and very much the normal use for this method should be CLI
+        # monitoring.
+        result = self.status(pk, detail=True)
+        last_poll = time.time()
+        timeout_check = 0
+        while result['status'] != 'successful':
+            # If the job has failed, we want to raise an Exception for that
+            # so we get a non-zero response.
+            if result['failed']:
+                if is_tty(outfile) and not settings.verbose:
+                    secho('\r' + ' ' * longest_string + '\n', file=outfile)
+                raise exc.JobFailure('Job failed.')
+
+            # Sanity check: Have we officially timed out?
+            # The timeout check is incremented below, so this is checking
+            # to see if we were timed out as of the previous iteration.
+            # If we are timed out, abort.
+            if timeout and timeout_check - start > timeout:
+                raise exc.Timeout('Monitoring aborted due to timeout.')
+
+            # If the outfile is a TTY, print the current status.
+            output = '\rCurrent status: %s%s' % (result['status'],
+                                                 '.' * next(dots))
+            if longest_string > len(output):
+                output += ' ' * (longest_string - len(output))
+            else:
+                longest_string = len(output)
+            if is_tty(outfile) and not settings.verbose:
+                secho(output, nl=False, file=outfile)
+
+            # Put the process to sleep briefly.
+            time.sleep(0.2)
+
+            # Sanity check: Have we reached our timeout?
+            # If we're about to time out, then we need to ensure that we
+            # do one last check.
+            #
+            # Note that the actual timeout will be performed at the start
+            # of the **next** iteration, so there's a chance for the job's
+            # completion to be noted first.
+            timeout_check = time.time()
+            if timeout and timeout_check - start > timeout:
+                last_poll -= interval
+
+            # If enough time has elapsed, ask the server for a new status.
+            #
+            # Note that this doesn't actually do a status check every single
+            # time; we want the "spinner" to spin even if we're not actively
+            # doing a check.
+            #
+            # So, what happens is that we are "counting down" (actually up)
+            # to the next time that we intend to do a check, and once that
+            # time hits, we do the status check as part of the normal cycle.
+            if time.time() - last_poll > interval:
+                result = self.status(pk, detail=True)
+                last_poll = time.time()
+                interval = min(interval * 1.5, max_interval)
+
+                # If the outfile is *not* a TTY, print a status update
+                # when and only when we make an actual check to job status.
+                if not is_tty(outfile) or settings.verbose:
+                    click.echo('Current status: %s' % result['status'],
+                               file=outfile)
+
+            # Wipe out the previous output
+            if is_tty(outfile) and not settings.verbose:
+                secho('\r' + ' ' * longest_string, file=outfile, nl=False)
+                secho('\r', file=outfile, nl=False)
+
+        # Return the job ID and other response data
+        answer = OrderedDict((
+            ('changed', True),
+            ('id', pk),
+        ))
+        answer.update(result)
+        # Make sure to return ID of resource and not update number
+        # relevant for project creation and update
+        answer['id'] = pk
+        return answer
+
+
+class ExeResource(MonitorableResource):
+    """Executable resource - defines status and cancel methods"""
+    abstract = True
+
+    @resources.command
+    @click.option('--detail', is_flag=True, default=False,
+                  help='Print more detail.')
+    def status(self, pk=None, detail=False, **kwargs):
+        """Print the current job status. This is used to check a running job.
+        You can look up the job with the same parameters used for a get
+        request."""
+        # Remove default values (anything where the value is None).
+        self._pop_none(kwargs)
+
+        # Search for the record if pk not given
+        if not pk:
+            job = self.get(include_debug_header=True, **kwargs)
+        # Get the job from Ansible Tower if pk given
+        else:
+            debug.log('Asking for job status.', header='details')
+            finished_endpoint = '%s%d/' % (self.endpoint, pk)
+            job = client.get(finished_endpoint).json()
+
+        # In most cases, we probably only want to know the status of the job
+        # and the amount of time elapsed. However, if we were asked for
+        # verbose information, provide it.
+        if detail:
+            return job
+
+        # Print just the information we need.
+        return {
+            'elapsed': job['elapsed'],
+            'failed': job['failed'],
+            'status': job['status'],
+        }
+
+    @resources.command
+    @click.option('--fail-if-not-running', is_flag=True, default=False,
+                  help='Fail loudly if the job is not currently running.')
+    def cancel(self, pk=None, fail_if_not_running=False, **kwargs):
+        """Cancel a currently running job.
+
+        Fails with a non-zero exit status if the job cannot be canceled.
+        You must provide either a pk or parameters in the job's identity.
+        """
+        # Search for the record if pk not given
+        if not pk:
+            existing_data = self.get(**kwargs)
+            pk = existing_data['id']
+
+        cancel_endpoint = '%s%d/cancel/' % (self.endpoint, pk)
+        # Attempt to cancel the job.
+        try:
+            client.post(cancel_endpoint)
+            changed = True
+        except exc.MethodNotAllowed:
+            changed = False
+            if fail_if_not_running:
+                raise exc.TowerCLIError('Job not running.')
+
+        # Return a success.
+        return {'status': 'canceled', 'changed': changed}
+
+
+class Resource(ResourceMethods):
+    """This is the parent class for all standard resources."""
+    abstract = True
+
+    @resources.command
+    @click.option('--fail-on-found', default=False,
+                  show_default=True, type=bool, is_flag=True,
+                  help='If used, return an error if a matching record already '
+                       'exists.')
+    @click.option('--force-on-exists', default=False,
+                  show_default=True, type=bool, is_flag=True,
+                  help='If used, if a match is found on unique fields, other '
+                       'fields will be updated to the provided values. If '
+                       'False, a match causes the request to be a no-op.')
+    def create(self, fail_on_found=False, force_on_exists=False, **kwargs):
+        """Create an object.
+
+        Fields in the resource's `identity` tuple are used for a lookup;
+        if a match is found, then no-op (unless `force_on_exists` is set) but
+        do not fail (unless `fail_on_found` is set).
+        """
+        return super(Resource, self).write(
+            create_on_missing=True,
+            fail_on_found=fail_on_found, force_on_exists=force_on_exists,
+            **kwargs
+        )
+
+    @resources.command(ignore_defaults=True)
+    @click.option('--create-on-missing', default=False,
+                  show_default=True, type=bool, is_flag=True,
+                  help='If used, and if options rather than a primary key are '
+                       'used to attempt to match a record, will create the '
+                       'record if it does not exist. This is an alias to '
+                       '`create --force-on-exists`.')
+    def modify(self, pk=None, create_on_missing=False, **kwargs):
+        """Modify an already existing object.
+
+        Fields in the resource's `identity` tuple can be used in lieu of a
+        primary key for a lookup; in such a case, only other fields are
+        written.
+
+        To modify unique fields, you must use the primary key for the lookup.
+        """
+        return self.write(pk, create_on_missing=create_on_missing,
+                          force_on_exists=True, **kwargs)
