@@ -1,4 +1,4 @@
-# Copyright 2014, Ansible, Inc.
+# Copyright 2015, Ansible, Inc.
 # Luke Sneeringer <lsneeringer@ansible.com>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,9 +20,11 @@ from tower_cli.utils import types
 class Resource(models.Resource):
     cli_help = 'Manage inventory within Ansible Tower.'
     endpoint = '/inventories/'
+    identity = ('organization', 'name')
 
     name = models.Field(unique=True)
     description = models.Field(required=False, display=False)
     organization = models.Field(type=types.Related('organization'))
-    variables = models.Field(type=models.File('r'), required=False,
-                             display=False)
+    variables = models.Field(
+        type=types.Variables(), required=False, display=False,
+        help_text='Inventory variables, use "@" to get from file.')
