@@ -1,4 +1,4 @@
-# Copyright 2014, Ansible, Inc.
+# Copyright 2015, Ansible, Inc.
 # Luke Sneeringer <lsneeringer@ansible.com>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -19,7 +19,7 @@ from tower_cli import models
 class Resource(models.Resource):
     cli_help = 'Manage users within Ansible Tower.'
     endpoint = '/users/'
-    unique_criterion = 'username'
+    identity = ('username',)
 
     username = models.Field(unique=True)
     password = models.Field(required=False, display=False)
@@ -27,3 +27,4 @@ class Resource(models.Resource):
     first_name = models.Field(required=False)
     last_name = models.Field(required=False)
     is_superuser = models.Field(required=False, type=bool)
+    is_system_auditor = models.Field(required=False, type=bool)
